@@ -8,6 +8,7 @@
 #pragma once
 
 #include <folly/Range.h>
+#include <optional>
 #include "watchman/fs/FileDescriptor.h"
 #include "watchman/watchman_string.h"
 
@@ -21,10 +22,6 @@ CaseSensitivity getCaseSensitivityForPath(const char* path);
 
 // Returns the name of the filesystem for the specified path
 w_string w_fstype(const char* path);
-w_string find_fstype_in_linux_proc_mounts(
+std::optional<w_string> find_fstype_in_linux_proc_mounts(
     std::string_view path,
     std::string_view procMountsData);
-
-inline bool is_edenfs_fs_type(w_string_piece fs_type) {
-  return fs_type == "edenfs" || fs_type.startsWith("edenfs:");
-}
