@@ -4,17 +4,21 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+# pyre-unsafe
+
 
 import pywatchman
 from watchman.integration.lib import WatchmanEdenTestCase
 
 
+def populate(repo):
+    repo.write_file(".watchmanconfig", '{"ignore_dirs":[".buckd"]}')
+    repo.write_file("hello", "hola\n")
+    repo.commit("initial commit.")
+
+
 class TestEdenUnmount(WatchmanEdenTestCase.WatchmanEdenTestCase):
     def test_eden_unmount(self) -> None:
-        def populate(repo):
-            repo.write_file(".watchmanconfig", '{"ignore_dirs":[".buckd"]}')
-            repo.write_file("hello", "hola\n")
-            repo.commit("initial commit.")
 
         root = self.makeEdenMount(populate)
         self.watchmanCommand("watch", root)
@@ -28,3 +32,12 @@ class TestEdenUnmount(WatchmanEdenTestCase.WatchmanEdenTestCase):
             self.watchmanCommand("query", root, {"fields": ["name"], "since": clock})
 
         self.assertRegex(str(ctx.exception), "unable to resolve root")
+
+    def test_eden_unmount_watch(self) -> None:
+        root = self.makeEdenMount(populate)
+        self.eden.unmount(root)
+
+        with self.assertRaises(pywatchman.CommandError) as ctx:
+            self.watchmanCommand("watch", root)
+
+        self.assertRegex(str(ctx.exception), "unable to resolve root")
