@@ -7,6 +7,7 @@
 
 #include <cpptoml.h>
 #include <fmt/core.h>
+#include <folly/ScopeGuard.h>
 #include <folly/String.h>
 #include <folly/futures/Future.h>
 #include <folly/io/async/AsyncSocket.h>
@@ -22,6 +23,7 @@
 #include <chrono>
 #include <iterator>
 #include <thread>
+#include "eden/common/utils/FSDetect.h"
 #include "eden/fs/service/gen-cpp2/StreamingEdenService.h"
 #include "watchman/ChildProcess.h"
 #include "watchman/Errors.h"
@@ -142,11 +144,27 @@ std::string resolveSocketPath(w_string_piece rootPath) {
 
   return *config->get_qualified_as<std::string>("Config.socket");
 #else
-  auto path = fmt::format("{}/.eden/socket", rootPath);
-  // It is important to resolve the link because the path in the eden mount
-  // may exceed the maximum permitted unix domain socket path length.
-  // This is actually how things our in our integration test environment.
-  return readSymbolicLink(path.c_str()).string();
+  try {
+    auto path = fmt::format("{}/.eden/socket", rootPath);
+    // It is important to resolve the link because the path in the eden mount
+    // may exceed the maximum permitted unix domain socket path length.
+    // This is actually how things our in our integration test environment.
+    return readSymbolicLink(path.c_str()).string();
+  } catch (const std::system_error& e) {
+    // When Eden fails during graceful takeover, the mount can exist, but it
+    // is disconnected. In this case, we can't use the eden watcher. Log this
+    // error and return no address - this will result makeThriftChannel failing
+    // with an AsyncSocketException.
+    log(DBG,
+        fmt::format(
+            "Failed to read EdenFS root when mount exists: {} ."
+            "{} appears to be a disconnected EdenFS mount. "
+            "Try running `eden doctor` to bring it back online and "
+            "then retry your watch.",
+            e.what(),
+            rootPath));
+    return std::string();
+  }
 #endif
 }
 
@@ -636,7 +654,8 @@ std::vector<NameAndDType> globNameAndDType(
     const std::string& mountPoint,
     const std::vector<std::string>& globPatterns,
     bool includeDotfiles,
-    bool splitGlobPattern = false) {
+    bool splitGlobPattern = false,
+    bool listOnlyFiles = false) {
   // TODO(xavierd): Once the config: "eden_split_glob_pattern" is rolled out
   // everywhere, remove this code.
   if (splitGlobPattern && globPatterns.size() > 1) {
@@ -651,6 +670,7 @@ std::vector<NameAndDType> globNameAndDType(
       params.globs() = std::vector<std::string>{globPattern};
       params.includeDotfiles() = includeDotfiles;
       params.wantDtype() = true;
+      params.listOnlyFiles() = listOnlyFiles;
       params.sync() = getSyncBehavior();
 
       globFutures.emplace_back(
@@ -669,6 +689,7 @@ std::vector<NameAndDType> globNameAndDType(
     params.globs() = globPatterns;
     params.includeDotfiles() = includeDotfiles;
     params.wantDtype() = true;
+    params.listOnlyFiles() = listOnlyFiles;
     params.sync() = getSyncBehavior();
 
     Glob glob;
@@ -680,7 +701,7 @@ std::vector<NameAndDType> globNameAndDType(
           "Thrift exception raised from EdenFS when globbing files: {} (errno {}, type {})\n",
           ex.what(),
           ex.getErrno(),
-          ex.getType());
+          fmt::underlying(ex.getType()));
       throw;
     } catch (const std::exception& ex) {
       logf(
@@ -849,12 +870,21 @@ class EdenView final : public QueryableView {
       bool includeDir = true) const {
     auto client = getEdenClient(thriftChannel_);
 
+    bool listOnlyFiles = false;
+    if (ctx->query->expr) {
+      listOnlyFiles =
+          ctx->query->expr->listOnlyFiles() == QueryExpr::ReturnOnlyFiles::Yes;
+    }
+    folly::stop_watch<std::chrono::microseconds> timer;
     auto fileInfo = globNameAndDType(
         client.get(),
         mountPoint_,
         globStrings,
         includeDotfiles,
-        splitGlobPattern_);
+        splitGlobPattern_,
+        listOnlyFiles);
+    ctx->edenGlobFilesDurationUs.store(
+        timer.elapsed().count(), std::memory_order_relaxed);
 
     // Filter out any ignored files
     filterOutPaths(fileInfo, ctx);
@@ -994,7 +1024,7 @@ class EdenView final : public QueryableView {
     thr.detach();
   }
 
-  void stopThreads() override {
+  void stopThreads(std::string_view /*reason*/) override {
     subscriberEventBase_.terminateLoopSoon();
   }
 
@@ -1069,7 +1099,7 @@ class EdenView final : public QueryableView {
     SCOPE_EXIT {
       // ensure that the root gets torn down,
       // otherwise we'd leave it in a broken state.
-      root->cancel();
+      root->cancel("eden subscriber thread exiting");
     };
 
     w_set_thread_name("edensub ", root->root_path.view());
@@ -1322,6 +1352,11 @@ class EdenView final : public QueryableView {
       // return a list of all possible matching files.
       return makeFreshInstance(ctx);
     }
+    folly::stop_watch<std::chrono::microseconds> timer;
+    SCOPE_EXIT {
+      ctx->edenChangedFilesDurationUs.store(
+          timer.elapsed().count(), std::memory_order_relaxed);
+    };
 
     try {
       return getAllChangesSinceStreaming(ctx);
@@ -1444,9 +1479,9 @@ std::shared_ptr<QueryableView> detectEden(
   }
 
 #else
-  if (!is_edenfs_fs_type(fstype) && fstype != "fuse" &&
+  if (!facebook::eden::is_edenfs_fs_type(fstype.string()) && fstype != "fuse" &&
       fstype != "osxfuse_eden" && fstype != "macfuse_eden" &&
-      fstype != "edenfs_eden") {
+      fstype != "edenfs_eden" && fstype != "fuse.edenfs") {
     // Not an active EdenFS mount.  Perhaps it isn't mounted yet?
     auto readme = fmt::format("{}/README_EDEN.txt", root_path);
     try {
@@ -1474,8 +1509,21 @@ std::shared_ptr<QueryableView> detectEden(
   }
 
   // Given that the readlink() succeeded, assume this is an Eden mount.
-  auto edenRoot =
-      readSymbolicLink(fmt::format("{}/.eden/root", root_path).c_str());
+  w_string edenRoot;
+  try {
+    edenRoot =
+        readSymbolicLink(fmt::format("{}/.eden/root", root_path).c_str());
+  } catch (const std::system_error& e) {
+    // When Eden fails during graceful takeover, the mount can exist, but it
+    // is disconnected. In this case, we can't use the eden watcher. Log this
+    // error and throw.
+    log(DBG, "Failed to read EdenFS root when mount exists: ", e.what());
+    RootNotConnectedError::throwf(
+        "{} appears to be a disconnected EdenFS mount. "
+        "Try running `eden doctor` to bring it back online and "
+        "then retry your watch",
+        root_path);
+  }
 
 #endif
   if (edenRoot != root_path) {
